@@ -16,17 +16,41 @@ typedef std::vector<std::string> stringVec;
 namespace StrUtil{
 
 
-inline bool isslash(int c){
-    return ( c == '/');
-}
+struct isSlash{
 
-inline bool isCrLf(int c){
-    return (( c == '\n') || (c == '\r'));
-}
+    bool operator()(char c) const{
+        return ( c == '/');
+    }
 
-inline bool isSpace(int c){
-    return static_cast<bool>(::isspace(static_cast<char>(c)));
-}
+    typedef char argument_type;
+};
+
+
+struct isCrLf{
+    bool operator()(char c) const{
+        return (( c == '\n') || (c == '\r'));
+    }
+
+    typedef char argument_type;
+};
+
+struct isSpace{
+    bool operator()(char c) const{
+        return ::isspace(c);
+    }
+
+    typedef char argument_type;
+
+};
+
+
+struct isHexa{
+    bool operator()(char c) const{
+        return (isdigit(c) || (isalpha(c) && ( tolower(c) >= 'a' && tolower(c) <= 'f')));
+    }
+
+    typedef char argument_type;
+};
 
 // predicates
 inline bool charEqCase(char c1, char c2){
@@ -35,6 +59,8 @@ inline bool charEqCase(char c1, char c2){
 
 
 
+
+
 /////////////////////////////
 // Parsing
 /////////////////////////////
@@ -58,7 +84,7 @@ inline std::string & remove(std::string & str, char c){
     return str;
 }
 
-
+std::string stringReplace(std::string str, const std::string & search, const std::string & replace);
 
 ///////////////////////////
 /// compare utils
@@ -66,6 +92,8 @@ inline std::string & remove(std::string & str, char c){
 // compare two C++ string case insensitive
 int compare_ncase(const std::string & str1, const std::string & str2);
 
+int compare_ncase(const std::string &str1, const std::string &str2, size_t max);
+
 int compare_ncase(const std::string & str1, off_t offset, size_t size, const char* cstr2);
 
 
@@ -82,29 +110,29 @@ inline size_t copy_std_string_to_buff(char* buffer, size_t max_size, const std::
 
 // trim from start
 template <typename Func>
-inline std::string &ltrim(std::string &s, Func  pred) {
-        s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, bool>(pred))));
+inline std::string &ltrim(std::string &s, const Func  & pred) {
+        s.erase(s.begin(), static_cast<std::string::iterator>(std::find_if(s.begin(), s.end(), std::not1(pred))));
         return s;
 }
 
 inline std::string &ltrim(std::string &s) {
-        return ltrim(s, isSpace);
+        return ltrim(s, isSpace());
 }
 
 // trim from end
 template <typename Func>
-inline std::string &rtrim(std::string &s, Func pred) {
-        s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, bool>(pred))).base(), s.end());
+inline std::string &rtrim(std::string &s, const Func & pred) {
+        s.erase(static_cast<std::string::iterator>(std::find_if(s.rbegin(), s.rend(), std::not1(pred)).base()), s.end());
         return s;
 }
 
 inline std::string &rtrim(std::string &s) {
-    return rtrim(s, isSpace);
+    return rtrim(s, isSpace());
 }
 
 // trim from both
 template <typename Func>
-inline std::string &trim(std::string &s, Func pred) {
+inline std::string &trim(std::string &s, const Func & pred) {
     return ltrim<Func>(rtrim<Func>(s, pred), pred);
 }
 
@@ -113,6 +141,12 @@ inline std::string &trim(std::string &s) {
 }
 
 
+inline std::string & toLower(std::string & str){
+    std::transform(str.begin(), str.end(), str.begin(), ::tolower);
+    return str;
+}
+
+
 }
 
 
