@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -47,18 +48,21 @@ public:
 
     // make collection
     virtual void makeCollection(IOChainContext & iocontext);
-    
+
     // move/rename resource
     virtual void move(IOChainContext & iocontext, const std::string & target_url);
 
     // get statInfo
     virtual StatInfo & statInfo(IOChainContext & iocontext, StatInfo & st_info);
 
+    // get quota info
+    virtual QuotaInfo & quotaInfo(IOChainContext & iocontext, QuotaInfo &info);
+
     virtual bool nextSubItem(IOChainContext &iocontext, std::string &entry_name, StatInfo &info);
 
 private:
 
-    Ptr::Scoped<DirHandle> directoryItem;
+    std::unique_ptr<DirHandle> directoryItem;
 };
 
 
@@ -71,6 +75,9 @@ public:
     // S3 + HTTP checksum computation
     virtual void checksum(IOChainContext & iocontext, std::string & checksm, const std::string & chk_algo);
 
+    // move/rename resource
+    virtual void move(IOChainContext & iocontext, const std::string & target_url);
+
     // get statInfo
     virtual StatInfo & statInfo(IOChainContext & iocontext, StatInfo & st_info);
 
@@ -81,10 +88,32 @@ public:
     virtual bool nextSubItem(IOChainContext &iocontext, std::string &entry_name, StatInfo &info);
 
 private:
-    Ptr::Scoped<DirHandle> directoryItem;
+    std::unique_ptr<DirHandle> directoryItem;
 
 };
 
+/// Handle all meta-data operations related to Azure
+class AzureMetaOps : public HttpIOChain{
+public:
+    AzureMetaOps();
+    virtual ~AzureMetaOps();
+
+    // Azure + HTTP checksum computation
+    //virtual void checksum(IOChainContext & iocontext, std::string & checksm, const std::string & chk_algo);
+
+    // get statInfo
+    virtual StatInfo & statInfo(IOChainContext & iocontext, StatInfo & st_info);
+
+    // make collection
+    //virtual void makeCollection(IOChainContext & iocontext);
+
+    // listing
+    virtual bool nextSubItem(IOChainContext &iocontext, std::string &entry_name, StatInfo &info);
+
+private:
+    std::unique_ptr<DirHandle> directoryItem;
+
+};
 
 
 /*
