@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,10 +19,10 @@
  *
 */
 
-
 #include <davix_internal.hpp>
 #include "davix_tool_params.hpp"
 #include "davix_tool_util.hpp"
+#include "davix_config_parser.hpp"
 #include <getopt.h>
 #include <string_utils/stringutils.hpp>
 #include <utils/davix_logger.hpp>
@@ -34,20 +35,34 @@ namespace Tool{
 const std::string scope_params = "Davix::Tools::Params";
 
 
-#define CAPATH_OPT          1000
-#define DEBUG_OPT           1001
-#define USER_LOGIN          1002
-#define USER_PASSWORD       1003
-#define DATA_CONTENT        1004
-#define S3_SECRET_KEY       1005
-#define S3_ACCESS_KEY       1006
-#define X509_PRIVATE_KEY    1007
-#define TRACE_OPTIONS       1008
-#define REDIRECTION_OPT     1009
-#define METALINK_OPT        1010
-#define CONN_TIMEOUT        1011
-#define TIMEOUT_OPS         1012
-#define RETRY_OPT           1013
+#define CAPATH_OPT             1000
+#define DEBUG_OPT              1001
+#define USER_LOGIN             1002
+#define USER_PASSWORD          1003
+#define DATA_CONTENT           1004
+#define S3_SECRET_KEY          1005
+#define S3_ACCESS_KEY          1006
+#define X509_PRIVATE_KEY       1007
+#define TRACE_OPTIONS          1008
+#define REDIRECTION_OPT        1009
+#define METALINK_OPT           1010
+#define CONN_TIMEOUT           1011
+#define TIMEOUT_OPS            1012
+#define RETRY_OPT              1013
+#define S3_LISTING_MODE        1014
+#define S3_MAX_KEYS            1015
+#define RETRY_DELAY_OPT        1016
+#define HAS_INPUT_FILE         1017
+#define THIRD_PT_COPY_MODE     1018
+#define DISABLE_LISTING_CAP    1019
+#define S3_REGION              1020
+#define S3_ALTERNATE           1021
+#define AZURE_KEY              1022
+#define S3_TOKEN               1023
+#define NO_100_CONTINUE        1024
+#define ACCEPTED_RETRY         1025
+#define ACCEPTED_RETRY_DELAY   1026
+#define GCLOUD_CRED_PATH       1027
 
 // LONG OPTS
 
@@ -61,9 +76,10 @@ const std::string scope_params = "Davix::Tools::Params";
 {"redirection", required_argument, 0, REDIRECTION_OPT }, \
 {"conn-timeout", required_argument, 0, CONN_TIMEOUT }, \
 {"retry", required_argument, 0, RETRY_OPT }, \
+{"retry-delay", required_argument, 0, RETRY_DELAY_OPT }, \
 {"timeout", required_argument, 0, TIMEOUT_OPS }, \
 {"trace", required_argument, 0, TRACE_OPTIONS }, \
-{"verbose", no_argument, 0,  0 }, \
+{"infile", required_argument, 0, HAS_INPUT_FILE }, \
 {"version", no_argument, 0, 'V'}
 
 #define SECURITY_LONG_OPTIONS \
@@ -74,6 +90,11 @@ const std::string scope_params = "Davix::Tools::Params";
 {"userpass", required_argument, 0, USER_PASSWORD}, \
 {"s3secretkey", required_argument, 0, S3_SECRET_KEY}, \
 {"s3accesskey", required_argument, 0, S3_ACCESS_KEY}, \
+{"s3region", required_argument, 0, S3_REGION}, \
+{"s3token", required_argument, 0, S3_TOKEN}, \
+{"azurekey", required_argument, 0, AZURE_KEY}, \
+{"s3alternate", no_argument, 0, S3_ALTERNATE}, \
+{"gcloud-creds", required_argument, 0, GCLOUD_CRED_PATH}, \
 {"insecure", no_argument, 0,  'k' }
 
 #define REQUEST_LONG_OPTIONS \
@@ -82,24 +103,45 @@ const std::string scope_params = "Davix::Tools::Params";
 {"verbose", no_argument, 0,  0 }
 
 #define LISTING_LONG_OPTIONS \
+{"s3-listing", required_argument, 0,  S3_LISTING_MODE }, \
+{"s3-maxkeys", required_argument, 0,  S3_MAX_KEYS }, \
+{"no-cap", required_argument, 0, DISABLE_LISTING_CAP}, \
 {"long-list", no_argument, 0,  'l' }
 
+#define GET_LONG_OPTIONS \
+{"accepted-retry", required_argument, 0, ACCEPTED_RETRY}, \
+{"accepted-retry-delay", required_argument, 0, ACCEPTED_RETRY_DELAY}
+
+#define PUT_LONG_OPTIONS \
+{"no-100-continue", no_argument, 0,  NO_100_CONTINUE }
+
+#define COPY_LONG_OPTIONS \
+{"copy-mode", required_argument, 0,  THIRD_PT_COPY_MODE }
+
 OptParams::OptParams() :
     params(),
     vec_arg(),
-    verbose(false),
     debug(false),
+    s3_delete_per_request(20),
+    threadpool_size(10),
     req_type(),
     help_msg(),
     cred_path(),
     priv_key(),
+    cred_pass(),
     output_file_path(),
     input_file_path(),
     userlogpasswd(),
+    userlogpasswd_through_cmdline(false),
     req_content(),
     aws_auth(),
+    aws_region(),
+    aws_token(),
+    aws_alternate(false),
     pres_flag(0),
-    shell_flag(0)
+    shell_flag(0),
+    has_input_file(false),
+    no_cap(false)
 {
 
 }
@@ -107,7 +149,7 @@ OptParams::OptParams() :
 
 static void option_abort(char** argv){
     std::cerr << argv[0] <<", Error: Wrong argument format\n";
-    std::cerr << "Try '" << argv[0] <<" --help' for more informations" << std::endl;
+    std::cerr << "Try '" << argv[0] <<" --help' for more information" << std::endl;
     exit(-1);
 }
 
@@ -133,7 +175,12 @@ static int set_header_field(const std::string & arg, OptParams & p, DavixError**
         DavixError::setupError(err, scope_params, StatusCode::InvalidArgument, " Invalid header field argument");
         return -1;
     }
-    p.params.addHeader(arg.substr(0,pos), arg.substr(pos+1));
+
+    dav_size_t value_pos = pos+1;
+    if(arg.size() > pos+1 && arg[pos+1] == ' ') {
+        value_pos = pos+2;
+    }
+    p.params.addHeader(arg.substr(0,pos), arg.substr(value_pos));
     return 0;
 }
 
@@ -174,6 +221,11 @@ static struct timespec parse_timeout(const std::string & opt, char** argv){
     return timelapse;
 }
 
+void parse_davix_config(OptParams &p, std::string url) {
+    if(davix_config_apply("~/.davixrc", p, url)) return;
+    if(davix_config_apply("~/.netrc", p, url)) return;
+}
+
 int parse_davix_options_generic(const std::string &opt_filter,
                                 const struct option* long_options,
                                 int argc, char** argv, OptParams & p, DavixError** err){
@@ -188,7 +240,7 @@ int parse_davix_options_generic(const std::string &opt_filter,
                 p.debug = true;
                 // max log level
                 setLogLevel(DAVIX_LOG_TRACE);
-                setLogScope(DAVIX_LOG_SCOPE_ALL | DAVIX_LOG_SCOPE_NEON | DAVIX_LOG_BODY | DAVIX_LOG_XML);
+                setLogScope(DAVIX_LOG_SCOPE_ALL ^ DAVIX_LOG_XML);
                 break;
             case 'E':
                  p.cred_path = SanitiseTildedPath(optarg);
@@ -200,17 +252,34 @@ int parse_davix_options_generic(const std::string &opt_filter,
                 if( set_header_field(optarg, p, err) <0)
                     return -1;
                 break;
+            case S3_LISTING_MODE:
+                {
+                    if(std::string(optarg).compare("flat")==0)
+                        p.params.setS3ListingMode(S3ListingMode::Flat);
+                    else if(std::string(optarg).compare("hierarchical")==0)
+                        p.params.setS3ListingMode(S3ListingMode::Hierarchical);
+                    else if(std::string(optarg).compare("semi")==0)
+                        p.params.setS3ListingMode(S3ListingMode::SemiHierarchical);
+                    break;
+                }
+            case S3_MAX_KEYS:
+                p.params.setS3MaxKey(atoi(optarg));
+                break;
             case CAPATH_OPT:
                 p.params.addCertificateAuthorityPath(optarg);
                 break;
             case USER_LOGIN:
                 p.userlogpasswd.first = optarg;
+                p.userlogpasswd_through_cmdline = true;
+                strncpy(optarg, "", strlen(optarg));
                 break;
             case X509_PRIVATE_KEY:
                 p.priv_key = SanitiseTildedPath(optarg).c_str();
                 break;
             case USER_PASSWORD:
                 p.userlogpasswd.second = optarg;
+                p.userlogpasswd_through_cmdline = true;
+                strncpy(optarg, "", strlen(optarg));
                 break;
             case DATA_CONTENT:
                 p.req_content = optarg;
@@ -218,11 +287,33 @@ int parse_davix_options_generic(const std::string &opt_filter,
             case RETRY_OPT:
                 p.params.setOperationRetry( parse_int(optarg, argv));
                 break;
+            case RETRY_DELAY_OPT:
+                p.params.setOperationRetryDelay( parse_int(optarg, argv));
+                break;
             case S3_ACCESS_KEY:
                 p.aws_auth.second = optarg;
+                strncpy(optarg, "", strlen(optarg));
                 break;
             case S3_SECRET_KEY:
                 p.aws_auth.first = optarg;
+                strncpy(optarg, "", strlen(optarg));
+                break;
+            case S3_REGION:
+                p.aws_region = optarg;
+                break;
+            case S3_TOKEN:
+                p.aws_token = optarg;
+                strncpy(optarg, "", strlen(optarg));
+                break;
+            case S3_ALTERNATE:
+                p.aws_alternate = true;
+                break;
+            case AZURE_KEY:
+                p.azure_key = optarg;
+                strncpy(optarg, "", strlen(optarg));
+                break;
+            case GCLOUD_CRED_PATH:
+                p.gcloud_creds_path = SanitiseTildedPath(optarg).c_str();
                 break;
             case 'l':
                 p.pres_flag |= LONG_LISTING_FLAG;
@@ -238,6 +329,7 @@ int parse_davix_options_generic(const std::string &opt_filter,
                 return 1;
             case TRACE_OPTIONS:
                 {
+                    p.debug = true;
                     setLogScope(0);
                     setLogScope(std::string(optarg));
                     setLogLevel(DAVIX_LOG_TRACE);
@@ -265,9 +357,45 @@ int parse_davix_options_generic(const std::string &opt_filter,
                 p.params.setOperationTimeout(&s);
                 break;
              }
+            case HAS_INPUT_FILE:{
+                p.has_input_file = true;
+                p.input_file_path = optarg;
+                }
+                break;
+            case THIRD_PT_COPY_MODE:{
+                if(std::string(optarg).compare("pull")==0)
+                    p.params.setCopyMode(CopyMode::Pull);
+                else if(std::string(optarg).compare("push")==0)
+                    p.params.setCopyMode(CopyMode::Push);
+                break;
+                }
+            case DISABLE_LISTING_CAP:{
+                p.no_cap = true;
+                break;
+                }
+            case 'r':{
+                p.params.setRecursiveMode(true);
+                p.threadpool_size = atoi(optarg);
+                break;
+                }
+            case 'n':{
+                p.s3_delete_per_request = (atoi(optarg));
+                break;
+                }
+            case NO_100_CONTINUE:{
+                p.params.set100ContinueSupport(false);
+                break;
+            }
+            case ACCEPTED_RETRY:
+                std::cout << "in accepted retry" << std::endl;
+                p.params.setAcceptedRetry(atoi(optarg));
+                break;
+            case ACCEPTED_RETRY_DELAY:
+                p.params.setAcceptedRetryDelay(atoi(optarg));
+                break;
             case '?':
-            printf(p.help_msg.c_str(), argv[0]);
-                return -1;
+                std::cout <<  p.help_msg;
+                exit(1);
           break;
             default:
                 option_abort(argv);
@@ -291,7 +419,7 @@ int parse_davix_options_generic(const std::string &opt_filter,
 
 
 int parse_davix_options(int argc, char** argv, OptParams & p, DavixError** err){
-    const std::string arg_tool_main= "P:x:H:E:X:o:kV";
+    const std::string arg_tool_main= "P:x:H:E:X:o:kVr:";
     const struct option long_options[] = {
         COMMON_LONG_OPTIONS,
         SECURITY_LONG_OPTIONS,
@@ -299,14 +427,19 @@ int parse_davix_options(int argc, char** argv, OptParams & p, DavixError** err){
         {0,         0,                 0,  0 }
      };
 
-    return parse_davix_options_generic(arg_tool_main, long_options,
+    if( parse_davix_options_generic(arg_tool_main, long_options,
                                        argc, argv,
-                                       p, err);
+                                       p, err) < 0) {
+        return -1;
+    }
+
+    parse_davix_config(p, p.vec_arg[0]);
+    return 0;
 }
 
 
 int parse_davix_ls_options(int argc, char** argv, OptParams & p, DavixError** err){
-    const std::string arg_tool_main= "P:x:H:E:vkVl";
+    const std::string arg_tool_main= "P:x:H:E:vkVlfr:";
     const struct option long_options[] = {
         COMMON_LONG_OPTIONS,
         SECURITY_LONG_OPTIONS,
@@ -321,15 +454,17 @@ int parse_davix_ls_options(int argc, char** argv, OptParams & p, DavixError** er
         option_abort(argv);
         return -1;
     }
+    parse_davix_config(p, p.vec_arg[0]);
     return 0;
 }
 
 
 int parse_davix_get_options(int argc, char** argv, OptParams & p, DavixError** err){
-    const std::string arg_tool_main= "P:x:H:E:o:OvkV";
+    const std::string arg_tool_main= "P:x:H:E:o:OvkVr:";
     const struct option long_options[] = {
         COMMON_LONG_OPTIONS,
         SECURITY_LONG_OPTIONS,
+        GET_LONG_OPTIONS,
         {0,         0,                 0,  0 }
      };
 
@@ -344,19 +479,19 @@ int parse_davix_get_options(int argc, char** argv, OptParams & p, DavixError** e
     if(p.vec_arg.size() == 2){
         p.output_file_path = p.vec_arg[1];
     }
+    parse_davix_config(p, p.vec_arg[0]);
     return 0;
 }
 
 int parse_davix_put_options(int argc, char** argv, OptParams & p, DavixError** err){
-    const std::string arg_tool_main= "P:x:H:E:o:vkV";
+    const std::string arg_tool_main= "P:x:H:E:o:vkVr:";
     const struct option long_options[] = {
         COMMON_LONG_OPTIONS,
         SECURITY_LONG_OPTIONS,
+        PUT_LONG_OPTIONS,
         {0,         0,                 0,  0 }
      };
 
-
-
     if( parse_davix_options_generic(arg_tool_main,
                                     long_options,
                                     argc,
@@ -368,14 +503,54 @@ int parse_davix_put_options(int argc, char** argv, OptParams & p, DavixError** e
         return -1;
     }
     p.input_file_path = p.vec_arg[0];
+    parse_davix_config(p, p.vec_arg[1]);
     return 0;
 }
 
+int parse_davix_copy_options(int argc, char** argv, OptParams & p, DavixError** err){
+    const std::string arg_tool_main= "P:x:H:E:X:o:kV";
+    const struct option long_options[] = {
+        COMMON_LONG_OPTIONS,
+        SECURITY_LONG_OPTIONS,
+        REQUEST_LONG_OPTIONS,
+        COPY_LONG_OPTIONS,
+        {0,         0,                 0,  0 }
+     };
 
-const std::string  & get_common_options(){
-    static const std::string s(
-            "  Common Options:\n"
+    if(parse_davix_options_generic(arg_tool_main, long_options,
+                                       argc, argv,
+                                       p, err) < 0) {
+        return -1;
+    }
+
+    parse_davix_config(p, p.vec_arg[0]);
+    return 0;
+}
+
+int parse_davix_rm_options(int argc, char** argv, OptParams & p, DavixError** err){
+    const std::string arg_tool_main= "P:x:H:E:X:o:kVr:n:";
+    const struct option long_options[] = {
+        COMMON_LONG_OPTIONS,
+        SECURITY_LONG_OPTIONS,
+        REQUEST_LONG_OPTIONS,
+        {0,         0,                 0,  0 }
+     };
+
+    if( parse_davix_options_generic(arg_tool_main, long_options,
+                                       argc, argv,
+                                       p, err) < 0) {
+        return -1;
+    }
+
+    parse_davix_config(p, p.vec_arg[0]);
+    return 0;
+}
+
+std::string get_common_options(){
+           return  "  Common Options:\n"
             "\t--conn-timeout TIME:      Connection timeout in seconds. default: 30\n"
+            "\t--retry NUMBER:           Number of retry attempts in case of an operation failure. default: 3\n"
+            "\t--retry-delay TIME:       Number of seconds to wait between retry attempts. default: 0\n"
             "\t--debug:                  Debug mode\n"
             "\t--header, -H:             Add a header field to the request\n"
             "\t--help, -h:               Display this help message\n"
@@ -384,8 +559,7 @@ const std::string  & get_common_options(){
             "\t--proxy, -x URL:          SOCKS5 proxy server URL. (Ex: socks5://login:pass@socks.example.org)\n"
             "\t--redirection OPT:        Transparent redirection support. value=yes|no. default=yes)\n"
             "\t--timeout TIME:           Global timeout for the operation in seconds. default: infinite\n"
-            "\t--trace:                  Specify one or more scpoes to trace. (Ex: --trace log level(optional),header,file)\n"
-            "\t--verbose:                Verbose mode\n"
+            "\t--trace:                  Specify one or more scopes to trace. (Ex: --trace log level(optional),header,file)\n"
             "\t--version, -V:            Display version\n"
             "  Security Options:\n"
             "\t--capath CA_PATH:         Add an additional certificate authority directory\n"
@@ -396,30 +570,38 @@ const std::string  & get_common_options(){
             "\t--userpass:               User password for login/password authentication\n"
             "\t--s3secretkey SEC_KEY:    S3 authentication: secret key\n"
             "\t--s3accesskey ACC_KEY:    S3 authentication: access key\n"
-            );
-    return s;
+            "\t--s3region REGION:        S3 region (optional - if passed, will authenticate using a v4 signature instead of v2)\n"
+            "\t--s3token TOKEN:          S3 security token - used along with STS temporary credentials\n"
+            "\t--s3alternate:            Pass this flag if you're using a path-based S3 URL\n"
+            "\t                          A path-based URL contains the bucket name in the path, ie https://s3-someregion.amazonaws.com/mybucket/file\n"
+            "\t--azurekey AZURE_KEY:     Azure authentication secret key\n"
+            "\t--gcloud-creds PATH:      Path to gcloud json credentials\n"
+            ;
 }
 
 
 
-const std::string  & get_base_description_options(){
-    static const std::string s("Usage: {} [OPTIONS ...] <url>\n"
-            );
-    return s;
+std::string  get_base_description_options(){
+    return "[OPTIONS ...] <url>\n"
+            ;
+}
+
+std::string  get_get_description_options(){
+    return "[OPTIONS ...] <url> [local_file]\n"
+            ;
 }
 
 
-const std::string  & get_put_description_options(){
-    static const std::string s("Usage: {} [OPTIONS ...] <local_file> <remote_file_url> \n"
-            );
-    return s;
+
+std::string get_put_description_options(){
+    return "[OPTIONS ...] <local_file> <remote_file_url> \n"
+            ;
 }
 
 
-const std::string  & get_copy_description_options(){
-    static const std::string s("Usage: {} [OPTIONS ...] <src_url> <dst_url>\n"
-            );
-    return s;
+std::string get_copy_description_options(){
+    return "[OPTIONS ...] <src_url> <dst_url>\n"
+            ;
 }
 
 }
