@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,13 +26,19 @@
 #include <utils/davix_logger.hpp>
 #include <string_utils/stringutils.hpp>
 
+#ifdef HAVE_ATOMIC
+#include <atomic>
+static std::atomic<int> internal_log_level(0);
+static std::atomic<int> internal_log_scope(DAVIX_LOG_SCOPE_ALL);
+#else
+#warning "Setting / getting davix loglevel is not thread-safe!"
+int internal_log_level = 0;
+int internal_log_scope = DAVIX_LOG_SCOPE_ALL;
+#endif
 
 const int BUFFER_SIZE =4096;
 const char* prefix = "DAVIX";
 
-static int internal_log_level = 0;
-static int internal_log_scope = DAVIX_LOG_SCOPE_ALL;
-
 static void (*_fhandler)(void* userdata, int mgs_level, const char* msg) = NULL;
 static void* _log_handler_userdata = NULL;
 
@@ -54,6 +61,7 @@ const char* SCOPE_CORE =    "core";
 const char* SCOPE_GRID =    "grid";
 const char* SCOPE_SOCKET =  "socket";
 const char* SCOPE_LOCKS =   "locks";
+const char* SCOPE_S3 =      "s3";
 const char* SCOPE_ALL =     "all";
 
 
@@ -70,6 +78,7 @@ const std::string davix_log_scope[] = {
     SCOPE_GRID,
     SCOPE_SOCKET,
     SCOPE_LOCKS,
+    SCOPE_S3,
     SCOPE_ALL
 };
 
@@ -138,6 +147,9 @@ void setLogScope(const std::string &scope){
                         mask |= DAVIX_LOG_LOCKS;
                         break;
                     case 12:
+                        mask |= DAVIX_LOG_S3;
+                        break;
+                    case 13:
                         mask |= DAVIX_LOG_SCOPE_ALL;
                         break;
                     default:
@@ -204,6 +216,9 @@ std::string getScopeName(int scope_mask){
         case DAVIX_LOG_LOCKS:
             scope_name = SCOPE_LOCKS;
             break;
+        case DAVIX_LOG_S3:
+            scope_name = SCOPE_S3;
+            break;
         default:
             scope_name = "Unknown";
             break;
@@ -265,8 +280,3 @@ extern "C"  void davix_set_log_handler( void (*fhandler)(void* userdata, int mgs
     _fhandler = fhandler;
     _log_handler_userdata = userdata;
 }
-
-
-
-
-
